#ifndef VTS_CLEAN_H
#define VTS_CLEAN_H

#include "includes.h"
#include <vector>

class CSubPic;

enum ProcessPhase
{
	Processing_Idle,
	Processing_Scanning_VOB,
	Processing_Done,
	Processing_Aborted,
	Processing_Applying,
	Processing_Reverting,
	Processing_Saving,
	Processing_ReTiming,
	Processing_UpdatingSRI,
	Processing_RunningOCR,
	Processing_FastReloading,
	Processing_Cropping,
};

typedef std::vector<int> intV;
typedef std::vector<unsigned char*> char16V;

class PGC_CLUT_Info
{
public:
	~PGC_CLUT_Info();
	uint	m_VCID;				// VCID that these cluts are associated with.
	int		m_NumCLUT;			// How many cluts.
	intV	m_LU;				// vector of LU
	intV	m_PGC;				// vector of PGG
	char16V	m_CLUT;				// vector of cluts.
	char	m_Languages[64];	// Languages of first clut.
};
typedef std::vector<PGC_CLUT_Info*> PGC_CLUT_InfoV;


typedef std::vector<CSubPic*> CSubPicPV;

typedef struct
{
	char			m_FileName[1024];		// Names of files to be scanned for SPUs.
	FILE*			m_FileFD;				// File Descr. of files to be scanned for SPUs.
	long			m_StartLBA;				// Cumulative start LBA for this file.
	long			m_SizeLBA;				// Size in lba of this file.
} FileInfo;

class VobuInfo
{
public:
	VobuInfo();
	int				m_OrigNavPackLBA;		// Cumulative LBA of navpack before any move occured.
	int				m_NavPackLBA;			// Cumulative LBA of navpack.
	int				m_PTSFirstFrame;		// PTS of first frame in VOBU
	int				m_EmptySlotCount;		// How many empty pack slots are in the VOBU.
	int				m_SizeLBA;				// Size of vobu in lba.
	uchar*			m_NavPtr;				// Pointer to the full navpack data.
} ;


typedef std::vector<VobuInfo> VobuInfoV;


class CSubPicEngine
{
public:
	CSubPicEngine();
	~CSubPicEngine();

	// Call this before you add any file via AddFile() for later scanning.
	virtual void	Reset();
	// Call this repeatedly to add files for later scanning.
	int				AddFile(const char* FileName);
	// After adding all the files via AddFile(), call this to start scanning.
	virtual void	ScanFilesForSPUs();
	// This scans the IFOs. It's called automatically in ScanFilesForSPUs();
	virtual void	ScanIFOs();
	// This opens a specific IFO, and uses the CLUT for the first PGC it can find.
	// Only use for a .sup file.
	int				OpenIFO(const char* FileName);
	// Call this to reload all the SPU from file. This does NOT rescan for subpic packs, instead
	// relies on the previous scan.
	virtual void	ReloadAllSPUs();
	// Call this to save all modified SPUs back to disk. Returns 0 if all OK, -1 if not.
	virtual void	SaveAllModifiedSPUs();

	// This saves all the spus in the pruned list to SupFileName, in the sup format. Returns 0 if all OK, -1 if not.
	virtual int		SaveSelectedAsSupFile(const char* SupFileName);

	// This is highly experimental and is just for testing! 
	int				LoadSUPFile(const char* SupFileName);

	// This crops the background on all the subpics in the pruned list.
	void			CropAllSubpics();

	// You can call this repeatedly to retrieve command info in a human-readable way. This applies
	// to m_SelectedSubPic.
	int				RetreiveNextCommand(char* Command);

	// This will be called each time an SPU is added during the scan. You can override it and do what
	// you will with it!
	virtual void	JustAddedSPU(int StreamIndex, int SPUIndex) {return;}
	virtual void	JustFoundStream(int StreamIndex) {return;}
	virtual void	JustFoundVCID(uint VCID) {return;}
	virtual void	JustFoundVideoFormat(){return;}

	// This adjusts the timing of the given subpic by DeltaPTS.
	int				ChangeTiming(int SubpicIndex, int DeltaPTS);

	// This saves subpic info to a file, so loading can happen really fast!
	int				SaveSubpicInfo(const char* FileName);
	int				LoadSubpicInfo(const char* FileName);
	// You can override this to be notified as soon as the files have been loaded, and before
	// SPUs have been parsed. I use that to open the file with the preview.
	virtual void	OnFileLoaded(const char* FileName) {return;}

	void			TestMove(int Index, float Seconds);
	// This moves the subpics currently in PrunedList.
	void			MoveSelection(float Seconds);
	void			MoveThisOne(int Index, float Seconds);

	// This creates m_PrunedList based on their stream ID, LBAs, or VCID.
	virtual int		PruneSPUList(int CurrentSelected, int StreamID = -1, int StartLBA = -1, int EndLBA = -1,
		ushort VOBID = 0, uchar CELLID = 0, int Modified = 0, char* Search = 0);

	// This looks into our array of cluts, and finds the first one that references VCID.
	uchar*			FindCLUT(ushort VOBID, uchar CELLID, char (*Language)[16]=0);

	float			PTSToSecs(int PTS);
	int				SecsToPTS(float Secs);

	ProcessPhase	m_Phase;					// What are we currently doing?
	int				m_PercentDone;

	int				m_NumVOBFiles;				// How many files we have to scan.
	FileInfo		m_FileInfos[10];
	VobuInfoV		m_VobuInfo;					// Vector of info on vobus.
	int*			m_InputLBA;					// Array of input navpack lbas.
	int*			m_OutputLBA;				// Array of output navpack lbas.
	bool			m_AbortNow;					// Set this to true to stop scanning.
	uchar			m_Buffer[SECTOR_SIZE];		// Temporary storage to read sectors.

	float			m_RemainingTimeSec;			// How long till we're done!

	CSubPicPV		m_SubPics[32];				// Two-dimensional array of pointers to SPU objects.
//												// m_SubPics[10][100] is the 100th SPU in stream 32+10.
	CSubPicPV		m_MasterList;				// List of all SPUs, in the order they were found.
	CSubPicPV		m_PrunedList;				// List of SPUs pruned according to filters.
	int				m_TotNumSPU;				// Overall number of found SPUs.
	int				m_PruneLength;				// Length of pruned list.
	bool			m_ListHasOCR;				// True if any of the subpics in the list have OCR data.
	uint			m_VCIDList[1024];			// List of found VCIDs (VOBID*128+CELLID).
	int				m_VCIDLength;
	PGC_CLUT_InfoV	m_CLUTV;
	char16V			m_AllCLUTS;					// vector of cluts.

	int				m_NumSubPics[32];			// How many SPUs per stream.
	bool			m_StreamCanMove[32];		// Indicates whether this spu stream can be retimed.
	int				m_TopStreamID;				// Highest stream ID for which we found an SPU.

	CSubPic			*m_SelectedSubPic;			// Subpic currently selected for display/modif.
	char			m_Version[512];

	int				m_PTSPerFrame;				// Frame duration in 90kHz ticks.
	int				m_PTSFirstFrame;			// PTS of first video frame in vob...
	uchar			m_AspectRatio;				// Aspect ratio. 1 for 1:1, 2 for 4:3, 3 for 16:9
												// and 4 for 2.21:1
	bool			m_CanSave;					// Will be false if VOBs are read-only
	bool			m_CanMove;					// Will be false if subpics can't be moved
	bool			m_CanReload;				// Indicates there's a VOB to reload from. Not always true (if loaded from sup).
	bool			m_NoVideo;					// Indicates that no video is available.
	bool			m_SupFileMode;				// Indicates that we're dealing with a SUP file, not vobs.

	bool			m_FoundILVU;				// Found ILVUs in the packs. This will limit moving.

protected:

	// A helper function used during loading.
	void			AfterLastPack(CSubPic* Temp, int StreamID);

	// I'm trying to be consistent here. 
	// - All the LBAs are absolute (cumulative) unless otherwise mentioned.
	// - The memory is origanized as:
	// [OrigLBA ....................... OrigLBA+Size];
	// OrigVOBU corresponds to OrigLBA;

	// MOST OF THIS IS FOR MOVING...
	// The area that moved is defined by [m_LeftLimitLBA ... m_RightLimitLBA] including the bounds.
	// m_MoveInc indicates if the area moved left (-1) or right (1) or didn't move (0).
	int				m_LeftLimitLBA;			// Left limit of zone that moved.
	int				m_RightLimitLBA;		// Right limit of zone that moved.
	int				m_MoveInc;				// -1 for move left, 1 for move right, 0 for no move.

	uchar*			m_MemoryPtr;			// Buffer used to move packs around.
	int				m_MemStartLBA;			// LBA corresponding to beginning of memory.
	int				m_MemSizeLBA;			// Size in LBA of loaded memory.
	int				m_MemAllocSize;			// Size of allocated memory (LBA).
	int				m_MemNeedsWrite;		// Indicates whether the memory needs to be written.
	int				m_MemOrigVOBU;

	int				m_SrcLBA;				// Original LBA of pack.
	int				m_DestLBABM;			// Destination LBA of pack, *before actual move*.
	int				m_DestLBAAM;			// Destination LBA of pack, *after actual move*.

	bool			m_PruneLoadWait;					// This is used as a semaphor to block updating some
											// of the lists while the file is loading.


	// Pass the original LBA, and the destination LBA *before the move*. This will set the move
	// area, and set m_SrcLBA, m_DestLBABM, m_DestLBAAM
	void			SetMoveAreaLimit(int SrcLBA, int DestLBA);
	// This returns true if the pack is within the area that was moved.
	bool			DidThisPackMove(int PackLBA);
	
	// Three helper functions to update the SRI (trick-play) pointers.
	int				UpdateThisPointer(int VobuIndex, int Offset);
	bool			ParseAndUpdatePointer1(uchar* Ptr, int Loc, int Dir, int VobuIndex);
	bool			ParseAndUpdatePointer2(uchar* Ptr, int Loc, int Dir, int VobuIndex);
	// This goes through all the navpacks, check each sri (trick play) offsets to see whether they
	// should be updated (because the target vobu has moved). 
	int				UpdateVobuSRI(bool JustCheck = false);

	// Pass a cumulative LBA, this returns the local LBA, and the FileIndex associated with it. 
	int				CumulToLocalLBA(int CumulLBA, int &FileIndex);
	int				LocalToCumulLBA(int LocalLBA, int FileIndex);
	// This loads NumLBA packs starting at the (cumulative) start into the memory.
	int				ReadPacksIntoMemory(int StartLBA, int NumLBA);
	int				WritePacksToFiles();
	// This function updates navpack pointers for a pack inserted at or removed from PackLBA.
	// The forward pointers, and 1st 2nd and 3rd references are updated if needed. Inc should be
	// +1 for an insertion and -1 for a deletion.
	void			UpdateNavPackPtrs(VobuInfo* Vobu, int PackLBA, int Inc);

	// This adjusts the pointers in the navpacks, and adjusts the data in m_VobuInfo, then
	// moves the pack according to the data set with SetMoveAreaLimit. OrigVOBU is the index in 
	// m_VobuInfo that correspond to the start of Ptr (m_LeftLimitLBA);
	int				MoveOnePack(int OrigVOBU);

	// This looks for an empty slot to put a pack. DestVobu points to the VOBU info of the vobu where
	// we'll be looking for an empty slot. We will search starting at LBAs after MinLBA (and after the
	// beginning of the vobu, of course).
	int				FindEmptySlot(VobuInfo* DestVobu, int MinLBA);
	int				FindEmptySlotBck(VobuInfo* DestVobu, int MinLBA);

	int				FindDestVobu(int SrcVobu, int PTS, int DeltaPTS);
	
	// This moves 1 pack according to the data set by SetMoveAreaLimit().
	int				MovePackInMem();

	// This function updates the subpic data to reflect the fact that packs where moved. In particular,
	// the navpack lba, and the spu's subpic pack pointers are updated.
	int				UpdateAllSubpics();

	// This isn't used.
	int				ReorderMasterList(int SrcSPUIndex);
	// This sorts the master spu list after a move. Puts them in the order of their first subpic pack.
	int				SortMasterLists();
	// A helper function.
	int				SortThisList(CSubPicPV* List, bool UpdateSPUNumber = false);
	// This updates the IFOs to reflect the fact that many packs changed locations. It updates the
	// VTS_C_ADT and VOBU_ADMAP tables.
	void			UpdateIFOAfterMove();

	// This checks that moving TheSubpic to the target vobu will not make it collide with another
	// one in the same subpic stream. Return -1 if all OK, or the subpic number if a collision is
	// detected.
	int				CheckForCollision(CSubPic* TheSubpic, int DestVobu);
} ;




#endif