/*
 *  This code is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  This code is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
*/

#include "StdAfx.h"
#include "Utils.h"
#include "string.h"
#include <sys/stat.h>
#include <io.h>
#include <shellapi.h>

// This is kind of ugly... I'm using a pool of static strings to store and return
// results for all the functions below. When a function needs a static string, it calls
// GetStr() which returns the next available one and increments the (circular) pointer.
// I used to declare a static char[] in each function, but that does not work if you use the function
// twice in the same call.
#define POOL_SIZE 16
char* GetStr()
{
	static char Pool[POOL_SIZE][1024];
	static int  PoolIndex = 0;
	char *Temp = Pool[PoolIndex++];
	PoolIndex %= POOL_SIZE;
	return Temp;
}
 
CSettings::CSettings(char* AppName)
{
	if(AppName == NULL)
		strcpy(m_AppName,GetFname(__argv[0]));
	else
		strcpy(m_AppName,AppName);
	
	// Create Ini file path.
	if(PathExists(getenv("APPDATA")))
		sprintf(m_IniFilePath,"%s/%s.ini",getenv("APPDATA"),m_AppName);
	else
		sprintf(m_IniFilePath,"./%s.ini",m_AppName);

	m_NumSettings = 0;
	ZeroMemory(m_Settings,1024*sizeof(ASetting));
}


CSettings::~CSettings()
{
}

void CSettings::LoadSettings(float* Value, char* KeyName, float DefValue)
{
	char Temp[MAX_PATH];
	GetPrivateProfileString(m_AppName,KeyName,"",Temp,512,m_IniFilePath);
	
	if(strcmp(Temp,"") == 0)
		*Value = DefValue;
	else
		sscanf(Temp,"%f",Value);
	m_Settings[m_NumSettings].m_Type = m_Float;
	m_Settings[m_NumSettings].m_Value = Value;
	strcpy(m_Settings[m_NumSettings].m_Key,KeyName);
	m_NumSettings++;
}	

void CSettings::LoadSettings(int* Value, char* KeyName, int DefValue)
{
	*Value = GetPrivateProfileInt(m_AppName,KeyName,DefValue,m_IniFilePath);
	m_Settings[m_NumSettings].m_Type = m_Int;
	m_Settings[m_NumSettings].m_Value = Value;
	strcpy(m_Settings[m_NumSettings].m_Key,KeyName);
	m_NumSettings++;
}	

void CSettings::LoadSettings(char* Value, char* KeyName, char* DefValue)
{
	GetPrivateProfileString(m_AppName,KeyName,DefValue,Value,512,m_IniFilePath);
	m_Settings[m_NumSettings].m_Type = m_String;
	m_Settings[m_NumSettings].m_Value = Value;
	strcpy(m_Settings[m_NumSettings].m_Key,KeyName);
	m_NumSettings++;
}	

void CSettings::LoadSettings(HWND HWnd, char* KeyName, bool NoResize)
{
	int x,y,cx,cy;
	char Temp[MAX_PATH];
	GetPrivateProfileString(m_AppName,KeyName,"0 0 0 0",Temp,MAX_PATH,m_IniFilePath);
	sscanf(Temp,"%d %d %d %d",&x,&y,&cx,&cy);
	x=max(x,0);y=max(y,0);cx=max(cx,0);cy=max(cy,0);
	UINT Flags = SWP_NOZORDER | ((NoResize) ? SWP_NOSIZE : 0);
	if(cx > 0 && cy > 0 && x >= 0 && y >= 0)
		::SetWindowPos(HWnd,NULL,x,y,cx,cy,Flags);

	m_Settings[m_NumSettings].m_Type = m_Hwnd;
	m_Settings[m_NumSettings].m_Value = HWnd;
	strcpy(m_Settings[m_NumSettings].m_Key,KeyName);
	m_NumSettings++;
}

void CSettings::DoSave(int i)
{
	char Temp[512];
	switch(m_Settings[i].m_Type)
	{
	case m_Int : 
		sprintf(Temp,"%d",*((int*)m_Settings[i].m_Value));
		WritePrivateProfileString(m_AppName,m_Settings[i].m_Key,Temp,m_IniFilePath);
		break;
	case m_Float : 
		sprintf(Temp,"%.12f",*((float*)m_Settings[i].m_Value));
		WritePrivateProfileString(m_AppName,m_Settings[i].m_Key,Temp,m_IniFilePath);
		break;
	case m_String : 
		WritePrivateProfileString(m_AppName,m_Settings[i].m_Key,(char*)m_Settings[i].m_Value,m_IniFilePath);
		break;
	case m_Hwnd : 
		RECT Rect;
		::GetWindowRect((HWND)(m_Settings[i].m_Value),&Rect);
		sprintf(Temp,"%d %d %d %d",Rect.left,Rect.top,Rect.right-Rect.left,Rect.bottom-Rect.top);
		WritePrivateProfileString(m_AppName,m_Settings[i].m_Key,Temp,m_IniFilePath);
		break;
	}
}

void CSettings::SaveSettings(void* Value)
{
	int i;
	for(i=0;i<m_NumSettings;i++)
	{
		if(m_Settings[i].m_Value == Value)
		{
			DoSave(i);
			return;
		}
	}
}

// This will save all the settings previously saved...	
void CSettings::SaveSettings()
{
	// Go through all our settings, and save them to the ini file...
	int i;
	for(i=0;i<m_NumSettings;i++)
	{
		DoSave(i);
	}
}

float CSettings::LoadSettings(char* KeyName, float DefValue)
{
	char Temp[MAX_PATH];
	GetPrivateProfileString(m_AppName,KeyName,"",Temp,512,m_IniFilePath);
	
	float Value;
	if(strcmp(Temp,"") == 0)
		Value = DefValue;
	else
		sscanf(Temp,"%f",&Value);
	return Value;
}	

int CSettings::LoadSettings(char* KeyName, int DefValue)
{
	return GetPrivateProfileInt(m_AppName,KeyName,DefValue,m_IniFilePath);
}	

char* CSettings::LoadSettings(char* KeyName, char* DefValue)
{
	char *Temp = GetStr();
	GetPrivateProfileString(m_AppName,KeyName,DefValue,Temp,512,m_IniFilePath);
	return Temp;
}	

void CSettings::SaveSettings(char* KeyName, float Value)
{
	char Temp[MAX_PATH];
	sprintf(Temp,"%.12f",Value);
	WritePrivateProfileString(m_AppName,KeyName,Temp,m_IniFilePath);
}	

void CSettings::SaveSettings(char* KeyName, int Value)
{
	char Temp[MAX_PATH];
	sprintf(Temp,"%d",Value);
	WritePrivateProfileString(m_AppName,KeyName,Temp,m_IniFilePath);
}	

void CSettings::SaveSettings(char* KeyName, char* Value)
{
	WritePrivateProfileString(m_AppName,KeyName,Value,m_IniFilePath);
}

void CSettings::OpenIniFile()
{
	ShellExecute(NULL, "open", m_IniFilePath, "", NULL, SW_SHOWNORMAL);
}

void OpenRSCFile(UINT RSCID, char* ext)
{
	LPTSTR Temp1 = MAKEINTRESOURCE(RSCID);
	HRSRC Temp2 = FindResource(NULL,Temp1,"BINARY");
	// We'll need to know the size!
	int Size = SizeofResource(NULL,Temp2);
	HGLOBAL Temp3 = LoadResource(NULL,Temp2);
	LPVOID Temp4 = LockResource(Temp3);
	// Put the file in the temp directory..
	char FileName[512];
	sprintf(FileName,"%s/ThisAppTemp%s",getenv("TEMP"),ext);
	FILE* fd = fopen(FileName,"wb");
	if(fd)
	{
		fwrite((char*)Temp4,1,Size,fd);
		fclose(fd);
	}
	// Now let's open the newly created file.
	int ret = (int)ShellExecute(NULL, "open", FileName, NULL, NULL, SW_SHOWNORMAL);
	if(ret < 32)
		AfxMessageBox("Can't open help!!");
}



HANDLE StartApp(const char* AppName, const char* Arguments, bool wait)
{
	PROCESS_INFORMATION ProcInfo;
	STARTUPINFO StrtupInfo;
	GetStartupInfo(&StrtupInfo);
	// I'm adding a space in front of the arguments, because if you forget, nothing works!
	// Also, if you pass " " as an argument, nothing works either! This is SOOOOO BROKEN!
	if(Arguments == NULL)
		CreateProcess(AppName,"", 0, 0, 1, CREATE_NEW_CONSOLE|NORMAL_PRIORITY_CLASS, 0, 0, &StrtupInfo, &ProcInfo);
	else
		CreateProcess(AppName,CatStr(" ",Arguments), 0, 0, 1, CREATE_NEW_CONSOLE|NORMAL_PRIORITY_CLASS, 0, 0, &StrtupInfo, &ProcInfo);
	if(wait)
	{
		WaitForSingleObject(ProcInfo.hProcess, INFINITE);
		CloseHandle(ProcInfo.hProcess);
		return 0;
	}
	return ProcInfo.hProcess;
}

// Concatenates str1 and str2 and returns result.
char* CatStr(const char* str1, const char* str2)
{
	char *Temp = GetStr();
	strcpy(Temp,str1);
	strcat(Temp,str2);
	return Temp;
}

// Concatenates str1, str2 and str3 and returns result.
char* CatStr(const char* str1, const char* str2, const char* str3)
{
	char *Temp = GetStr();
	strcpy(Temp,str1);
	strcat(Temp,str2);
	strcat(Temp,str3);
	return Temp;
}

// Find extension in srt1.
char* GetExt(const char* str1)
{
	char *Temp = GetStr();
	_splitpath(str1,0,0,0,Temp);
	return Temp;
}

// Find filename in full path. Ex: for str1 = "C:/temp/foo.bit" returns foo.
char* GetFname(const char* str1)
{
	char drive[_MAX_DRIVE];
	char dir[_MAX_DIR];
	char *fname = GetStr();
	char ext[_MAX_EXT];
	_splitpath( str1, drive, dir, fname, ext );
	return fname;
}

// Find filename+ext in full path. Ex: for str1 = "C:/temp/foo.bit" returns foo.bit.
char* GetFnameExt(const char* str1)
{
	char drive[_MAX_DRIVE];
	char dir[_MAX_DIR];
	char *fname = GetStr();
	char ext[_MAX_EXT];
	_splitpath( str1, drive, dir, fname, ext );
	strcat(fname,ext);
	return fname;
}

// Find full directory path. Ex: for str1 = "C:/temp/foo.bit" returns C:/temp/.
char* GetDirPath(const char* str1)
{
	char *drive = GetStr();
	char dir[_MAX_DIR];
	char fname[_MAX_FNAME];
	char ext[_MAX_EXT];
	_splitpath( str1, drive, dir, fname, ext );
	strcat(drive,dir);
	return drive;
}

// Change extension. Ex: for str1 = "C:/temp/foo.bit" and Ext = ".vob" returns C:/temp/foo.vob
char* ChangeExt(const char* str1, const char* Ext)
{
	char *drive = GetStr();
	char dir[_MAX_DIR];
	char fname[_MAX_FNAME];
	char ext[_MAX_EXT];
	_splitpath( str1, drive, dir, fname, ext );
	strcat(drive,dir);
	strcat(drive,fname);
	strcat(drive,Ext);
	return drive;
}

bool PathExists(const char* str1)
{
	struct _stat StatBuff;
	return _stat(str1, &StatBuff) == 0;
}

bool IsDir(const char* str1)
{
	struct _stat StatBuff;
	if(_stat(str1, &StatBuff) != 0)
		return false;
	return StatBuff.st_mode & _S_IFDIR != 0 ;
}

// Concatenates str1 and str2 and returns result.
char* CatPath(const char* str1, const char* str2)
{
	char *Temp = GetStr();
	strcpy(Temp,str1);
	// Adding / or \ to make path, if needed.
	if(str2[0] != '\\' &&  str2[0] != '/' && str1[strlen(str1)-1] != '\\' && 
		str1[strlen(str1)-1] != '/')
		strcat(Temp,"/");
	strcat(Temp,str2);
	return Temp;
}

// Concatenates str1, str2 and str3 and returns result.
char* CatPath(const char* str1, const char* str2, const char* str3)
{
	char *Temp = GetStr();
	strcpy(Temp,str1);
	// Adding / or \ to make path, if needed.
	if(str2[0] != '\\' &&  str2[0] != '/' && str1[strlen(str1)-1] != '\\' && 
		str1[strlen(str1)-1] != '/')
		strcat(Temp,"/");
	strcat(Temp,str2);
	if(str3[0] != '\\' &&  str3[0] != '/' && str2[strlen(str2)-1] != '\\' && 
		str2[strlen(str2)-1] != '/')
		strcat(Temp,"/");
	strcat(Temp,str3);
	return Temp;
}

unsigned long GetFileSizekB(const char* str1)
{
	struct _stati64 StatBuff;
	if(_stati64(str1, &StatBuff) != 0)
		return 0;
	return (unsigned long)(StatBuff.st_size / 1024);
}

bool IsEmpty(const char* str1)
{
	struct _stati64 StatBuff;
	if(_stati64(str1, &StatBuff) != 0)
		return true;
	return (StatBuff.st_size == 0);
}


void EmptyDir(const char* str1)
{
	struct _finddata_t FileInfo;
	int FindIt = _findfirst( CatPath(str1,"*"), &FileInfo );
	if(strcmp(FileInfo.name,".") != 0 && strcmp(FileInfo.name,"..") != 0)
		remove(CatPath(str1,FileInfo.name));
	while(_findnext(FindIt, &FileInfo ) == 0)
	{
		if(strcmp(FileInfo.name,".") != 0 && strcmp(FileInfo.name,"..") != 0)
			remove(CatPath(str1,FileInfo.name));
	}
	_findclose( FindIt );
}

char* GetCurrentDir()
{
	char *Temp = GetStr();
	if(GetCurrentDirectory(1024,Temp) == 0)
		return 0;
	return Temp;
}


void ShutDownComputer()
{
	HANDLE hToken; 
	TOKEN_PRIVILEGES tkp; 

	// Get a token for this process. 
	if (!OpenProcessToken(GetCurrentProcess(), 
		TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken)) 
	  return; 
	// Get the LUID for the shutdown privilege. 
	LookupPrivilegeValue(NULL, SE_SHUTDOWN_NAME, 
		&tkp.Privileges[0].Luid); 
	tkp.PrivilegeCount = 1;  // one privilege to set    
	tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED; 
	// Get the shutdown privilege for this process. 
	AdjustTokenPrivileges(hToken, FALSE, &tkp, 0, 
		(PTOKEN_PRIVILEGES)NULL, 0); 
	// now, shutdown...
	ExitWindowsEx(EWX_POWEROFF | EWX_FORCE,0); 
//		SHTDN_REASON_MAJOR_OPERATINGSYSTEM | SHTDN_REASON_FLAG_PLANNED);
}

char* GetIFO(const char* VTSName)
{
	char *Temp = GetStr();
	// Change extension to IFO. If it's not video_ts, then change number to a '0'.
	strcpy(Temp,ChangeExt(VTSName,".IFO"));
	if(strcmpi(GetFname(VTSName),"video_ts") != 0)
		Temp[strlen(Temp)-5]='0';
	return Temp;
}

char* GetBUP(const char* VTSName)
{
	char *Temp = GetStr();
	// Change extension to BUP. If it's not video_ts, then change number to a '0'.
	strcpy(Temp,ChangeExt(VTSName,".BUP"));
	if(strcmpi(GetFname(VTSName),"video_ts") != 0)
		Temp[strlen(Temp)-5]='0';
	return Temp;
}